// simple graphic animation

#include <math.h>
#include "pos.h"
#include "grproc.h"

#ifdef _DEBUG
// set debug infos to debug with visual studio and emulator.
void SetDebugString(const char **CmdLine, const char **ExePath, const char **RunPath)
{
  *CmdLine = "";                       // command line
  *ExePath = "";                       // simulated path of executable
  *RunPath = "";                       // running path
}
#endif

vec2i WinPos, WinSize;

#define FPS 30
#define _2PI (2.0f*3.14159265359f)

// main
int main(int argc, char **argv)
{
  u32 col;
  u8 r, g, b;
  float tx, ty, a, da, x, y, cx, cy, wx, wy, px, py;

  if (!InitGrProc())
    return 0;

  // define a drawing window
  WinSize.x = Fb.x / 2;
  WinSize.y = Fb.y / 2;
  WinPos.x = (Fb.x - WinSize.x) / 2;
  WinPos.y = (Fb.y - WinSize.y) / 2;

  da = _2PI / 1000.0f;
  wx = (float)WinSize.x/2;
  wy = (float)WinSize.y/2;
  cx = (float)(WinPos.x + WinSize.x/2);
  cy = (float)(WinPos.y + WinSize.y/2);

  tx = ty = 0;
  r = g = b = 0;
  for (;;)
  {
    Kernel.Time.Sleep(1000/FPS);                 // set framerate

    if (Kernel.Kbd.KbdHit())                     // exit on key pressed
      break;

    tx += 0.01f;
    if (tx >= _2PI) tx -= _2PI;
    px = (float)sin(tx)*3;

    ty += 0.03f;
    if (ty >= _2PI) ty -= _2PI;
    py = (float)sin(ty)*4;

    r += 1;
    g += 3;
    b += 4;
    col = GetColor(r, g, b);

    ClearFb(&WinPos, &WinSize, 0);
    for (a=0; a<_2PI; a+=da)
    {
      x = cx - wx*(float)sin(a*px)*(float)cos(a);
      y = cy - wy*(float)cos(a*py)*(float)sin(a);
      DrawPixel((s32)x, (s32)y, col);
    }
    Kernel.Disp.BlitRect(WinPos.x, WinPos.y, WinSize.x, WinSize.y);
  }

  // clear at exit
  ClearFb(&WinPos, &WinSize, 0);
  Kernel.Disp.BlitRect(WinPos.x, WinPos.y, WinSize.x, WinSize.y);

  return 1;
}
