// simple graphic program, draw sinus curve
#include <math.h>
#include "pos.h"

#ifdef _DEBUG
// set debug infos to debug with visual studio and emulator.
void SetDebugString(const char **CmdLine, const char **ExePath, const char **RunPath)
{
  *CmdLine = "";                       // command line
  *ExePath = "";                       // simulated path of executable
  *RunPath = "";                       // running path
}
#endif

// frame buffer informations
struct FbInf
{
  s32 x;                               // x resolution
  s32 y;                               // y resolution
  s32 Bpp;                             // color depth
  s32 FBReserved;                      // reserved ram size
  void *pScreen;                       // pointer to screen (not vram)
  void *pVRam;                         // pointer to vram, NULL if direct access is not possible
} Fb;

// test if pixel is into screen
#define InScreen(x, y) (x >= 0) && (x < Fb.x) && (y >= 0) && (y < Fb.y)

// color in RGB 232
static void DrawPixel8(int x, int y, int Color)
{
  if (InScreen(x, y))
  {
    u8 *p = (u8 *)Fb.pScreen;
    p[y*Fb.x + x] = (u8)Color;
  }
}

// color in RGB 565
static void DrawPixel16(int x, int y, int Color)
{
  if (InScreen(x, y))
  {
    u16 *p = (u16 *)Fb.pScreen;
    p[y*Fb.x + x] = (u16)Color;
  }
}

// color in RGB 888
static void DrawPixel32(int x, int y, int Color)
{
  if (InScreen(x, y))
  {
    u32 *p = (u32 *)Fb.pScreen;
    p[y*Fb.x + x] = (u32)Color;
  }
}

static void (*DrawPixel)(int x, int y, int Color);

// main
int main(int argc, char **argv)
{
  int x, y;
  float a, k, b;
  DrawPixel = NULL;

  // get frame buffer infos
  Fb.pScreen = Kernel.Disp.GetFrameBuffer(&Fb.x, &Fb.y, &Fb.Bpp, &Fb.FBReserved, &Fb.pVRam);
  printf("Screen size = %d x %d, Bpp = %d\n", Fb.x, Fb.y, Fb.Bpp);

  if (Fb.pScreen)
  {
    // define a suitable pixel draw proc
    if (Fb.Bpp == 8)
      DrawPixel = DrawPixel8;
    else
    if (Fb.Bpp == 16)
      DrawPixel = DrawPixel16;
    else
    if (Fb.Bpp == 32)
      DrawPixel = DrawPixel32;
  }

  if (!DrawPixel)
  {
    printf("ERROR: no suitable graphic mode found.\n");  // we are in text mode ?
    return 0;
  }

  // draw a sinus
  a = 0;                                        // angle
  k = 3.1415926f*2.0f/(float)Fb.x;              // angle increment
  b = (float)Fb.y*0.5f;                         // half screen size
  
  for (x=0; x<Fb.x; x++, a+=k)
  {
    y = (int)(b - b*sin(a));
    DrawPixel(x, y, 0xffffff);                  // draw in white
  }

  // blit result
  Kernel.Disp.BlitAll();
  return 1;
}
