#include "pos.h"

#ifdef _DEBUG
// set debug infos to debug with visual studio and emulator.
void SetDebugString(const char **CmdLine, const char **ExePath, const char **RunPath)
{
  *CmdLine = "";                       // command line
  *ExePath = "";                       // simulated path of executable
  *RunPath = "";                       // running path
}
#endif

#define INT_ALLOC (1*1024*1024)        // 4 mo

struct Tmes
{
  u64 TscElapsed;
  u64 Transferred;
};

// frame buffer informations
struct FbInf
{
  s32 x;                               // x resolution
  s32 y;                               // y resolution
  s32 Bpp;                             // color depth
  s32 FBReserved;                      // reserved ram size
  void *pScreen;                       // pointer to screen
  void *pVRam;                         // pointer to vram
};

s32 main(s32 argc, char **argv)
{
  u64 t0, t1, tt, tmax;
  u32 i, *p, Sum, FbSize;
  struct Tmes MesList[4];
  s32 *Mem, *MemEnd;
  struct FbInf Fb;
  const char *Labels[4] = {"RamWR ", "RamRD ", "VRamWR", "VRamRD"};

  Mem = (s32 *)malloc(INT_ALLOC*4);
  if (!Mem)
  {
    printf("ERROR: Test require %u mb of free memory to execute\n", INT_ALLOC*4);
    return 0;
  }
  MemEnd = Mem + INT_ALLOC;

  printf("Testing ram speed, test time is 12 seconds..\n");
  
  Sum = 0;
  tt = (u64)Kernel.Time.TSC_One_ms*3000;

  // --------------------------------------------
  // test ram write speed
  t0 = Kernel.Time.ReadTsc();
  tmax = t0 + tt;
  for (i=0;;i++)
  {
    for (p = Mem; p<MemEnd; p++)
      *p = Sum++;
    t1 = Kernel.Time.ReadTsc();
    if (t1 > tmax)
    {
      MesList[0].TscElapsed = t1 - t0;
      MesList[0].Transferred = INT_ALLOC*4*i;
      break;
    }
  }

  // --------------------------------------------
  // test ram read speed
  t0 = Kernel.Time.ReadTsc();
  tmax = t0 + tt;
  for (i=0;;i++)
  {
    for (p = Mem; p<MemEnd; p++)
      Sum -= *p;
    t1 = Kernel.Time.ReadTsc();
    if (t1 > tmax)
    {
      MesList[1].TscElapsed = t1 - t0;
      MesList[1].Transferred = INT_ALLOC*4*i;
      break;
    }
  }

  free(Mem);

  // --------------------------------------------
  // test vram write speed
  Fb.pScreen = Kernel.Disp.GetFrameBuffer(&Fb.x, &Fb.y, &Fb.Bpp, &Fb.FBReserved, &Fb.pVRam);

  if (Fb.pVRam)                        // NULL on emulator or if vram direct access is not available
  {
    Mem = (s32 *)Fb.pVRam;
    FbSize = Fb.x * Fb.y * (Fb.Bpp / 8);
    MemEnd = Mem + (FbSize >> 2);

    t0 = Kernel.Time.ReadTsc();
    tmax = t0 + tt;
    for (i=0;;i++)
    {
      for (p = Mem; p<MemEnd; p++)
        *p = Sum++;
      t1 = Kernel.Time.ReadTsc();
      if (t1 > tmax)
      {
        MesList[2].TscElapsed = t1 - t0;
        MesList[2].Transferred = FbSize*i;
        break;
      }
    }

    // --------------------------------------------
    // test ram read speed
    t0 = Kernel.Time.ReadTsc();
    tmax = t0 + tt;
    for (i=0;;i++)
    {
      for (p = Mem; p<MemEnd; p++)
        Sum -= *p;
      t1 = Kernel.Time.ReadTsc();
      if (t1 > tmax)
      {
        MesList[3].TscElapsed = t1 - t0;
        MesList[3].Transferred = FbSize*i;
        break;
      }
    }
  }

  // --------------------------------------------
  // display results
  Kernel.Disp.ClearScreen();
  for (i=0; i<4; i++)
  {
    if ((i >= 2) && !Fb.pVRam)
      printf("%s speed  = (skipped)\n", Labels[i]);
    else
    {
      u64 Transferred = MesList[i].Transferred;
      u64 TscElapsedms = MesList[i].TscElapsed / Kernel.Time.TSC_One_ms;
      u64 s = (u32)(Transferred*1000 / (TscElapsedms*1024));
      printf("%s speed  = %u Kb/s\n", Labels[i], s);
    }
  }
  return Sum | 1;
}

