// kernel header interface, for user applications

#ifdef  __cplusplus
extern "C" {
#endif

// ----------------------------------------------
// keyboard

// modifiers for ascii codes
#define K_CTRL(c) ((c) & 0x1F)                   // Map to control code
#define K_ALT(c)  ((c) | 0x80)                   // Set eight bit (ALT)

#define KEY_RETURN 0x0d
#define KEY_ESCAPE 0x1b
#define KEY_BACK   0x08
#define KEY_TAB    0x09
#define KEY_DEL    0x7f

#define KEY_HOME   (0x01 + 0x0100)
#define KEY_END    (0x02 + 0x0100)
#define KEY_UP     (0x03 + 0x0100)
#define KEY_DOWN   (0x04 + 0x0100)
#define KEY_LEFT   (0x05 + 0x0100)
#define KEY_RIGHT  (0x06 + 0x0100)
#define KEY_PGUP   (0x07 + 0x0100)
#define KEY_PGDN   (0x08 + 0x0100)
#define KEY_MID    (0x09 + 0x0100)
#define KEY_NMIN   (0x0A + 0x0100)
#define KEY_PLUS   (0x0B + 0x0100)
#define KEY_INSRT  (0x0C + 0x0100)

// Function keys
#define KEY_F1     (0x10 + 0x0100)
#define KEY_F2     (0x11 + 0x0100)
#define KEY_F3     (0x12 + 0x0100)
#define KEY_F4     (0x13 + 0x0100)
#define KEY_F5     (0x14 + 0x0100)
#define KEY_F6     (0x15 + 0x0100)
#define KEY_F7     (0x16 + 0x0100)
#define KEY_F8     (0x17 + 0x0100)
#define KEY_F9     (0x18 + 0x0100)
#define KEY_F10    (0x19 + 0x0100)
#define KEY_F11    (0x1A + 0x0100)
#define KEY_F12    (0x1B + 0x0100)

// ctrl + key
#define KEY_CHOME  (0x01 + 0x0200)
#define KEY_CEND   (0x02 + 0x0200)
#define KEY_CUP    (0x03 + 0x0200)
#define KEY_CDOWN  (0x04 + 0x0200)
#define KEY_CLEFT  (0x05 + 0x0200)
#define KEY_CRIGHT (0x06 + 0x0200)
#define KEY_CPGUP  (0x07 + 0x0200)
#define KEY_CPGDN  (0x08 + 0x0200)
#define KEY_CMID   (0x09 + 0x0200)
#define KEY_CNMIN  (0x0A + 0x0200)
#define KEY_CPLUS  (0x0B + 0x0200)
#define KEY_CINSRT (0x0C + 0x0200)

// Ctrl+Fn
#define KEY_CF1    (0x10 + 0x0200)
#define KEY_CF2    (0x11 + 0x0200)
#define KEY_CF3    (0x12 + 0x0200)
#define KEY_CF4    (0x13 + 0x0200)
#define KEY_CF5    (0x14 + 0x0200)
#define KEY_CF6    (0x15 + 0x0200)
#define KEY_CF7    (0x16 + 0x0200)
#define KEY_CF8    (0x17 + 0x0200)
#define KEY_CF9    (0x18 + 0x0200)
#define KEY_CF10   (0x19 + 0x0200)
#define KEY_CF11   (0x1A + 0x0200)
#define KEY_CF12   (0x1B + 0x0200)

// Alt+Fn
#define KEY_AF1    (0x10 + 0x0800)
#define KEY_AF2    (0x11 + 0x0800)
#define KEY_AF3    (0x12 + 0x0800)
#define KEY_AF4    (0x13 + 0x0800)
#define KEY_AF5    (0x14 + 0x0800)
#define KEY_AF6    (0x15 + 0x0800)
#define KEY_AF7    (0x16 + 0x0800)
#define KEY_AF8    (0x17 + 0x0800)
#define KEY_AF9    (0x18 + 0x0800)
#define KEY_AF10   (0x19 + 0x0800)
#define KEY_AF11   (0x1A + 0x0800)
#define KEY_AF12   (0x1B + 0x0800)

// key states, parameter for keyboard GetKeyStates()
enum eKeyState
{
  KEY_ST_ALT_L = 0,                              // left alt key state
  KEY_ST_ALT_R,                                  // right alt key state
  KEY_ST_ALT,                                    // either alt key
  KEY_ST_CTRL_L,                                 // left control key state
  KEY_ST_CTRL_R,                                 // right control key state
  KEY_ST_CTRL,                                   // either control key
  KEY_ST_SHIFT_L,                                // left shift key state
  KEY_ST_SHIFT_R,                                // right shift key state
  KEY_ST_SHIFT,                                  // either shift key
  KEY_ST_NUM_DOWN,                               // num lock key depressed
  KEY_ST_CAPS_DOWN,                              // caps lock key depressed
  KEY_ST_SCRL_DOWN,                              // scroll lock key depressed
  KEY_ST_LOCKS,                                  // lock keys state
};

// ----------------------------------------------
// mouse

// mouse events
#define MEVENT_LBUTT 1
#define MEVENT_RBUTT 2
#define MEVENT_MBUTT 4
#define MEVENT_MOVED 8
#define MEVENT_WHELL 16

// buttons bits
#define MOUSE_LBUTT 1
#define MOUSE_RBUTT 2
#define MOUSE_MBUTT 4

// mouse cursor shape
enum ePosMouseCursor
{
  MC_ARROW = 0,                                  // base arrow
  MC_SIZEX,                                      // size horizontal
  MC_SIZEY,                                      // size vertical
  MC_SIZEXYBR,                                   // x/y bottom right
  MC_SIZEXYTR,                                   // x/y top right
  MC_SIZEALL,                                    // all directions
  MC_HIDDEN,                                     // no cursor, is hidden (but active). must be at END OF LIST
};

// ----------------------------------------------
// files

#define FN_LEN_MAX 256                           // file name len max (255 + ending 0)

// file info flags
enum eFF_FLAGS
{
  FF_READONLY  = 0x1,
  FF_HIDDEN    = 0x2,
  FF_SYSTEM    = 0x4,
  FF_LABEL     = 0x8,
  FF_DIRECTORY = 0x10,
  FF_ARCHIVE   = 0x20,
};

// file informations for findfirst/findnext
struct TFileInfo
{
  enum eFF_FLAGS Flags;                          // attributes flags
  const char *_pName;                            // name of file (temporary valid until next findnext command)
  u32 Size;                                      // size
  u16 Date;                                      // date
  u16 Time;                                      // time
};

// file handle
typedef struct { s32 reserved; } hFILE;

// findfirst handle
typedef struct { s32 reserved; } hFFIND;

// file access mode
#define O_RDONLY  0
#define O_WRONLY  1
#define O_RDWR    2
#define O_CREAT   0x0100                          // Create the file if it does not exist.
#define O_TRUNC   0x0200                          // (ignored) Truncate the file if it does exist.
#define O_EXCL    0x0400                          // (ignored) Open only if the file does not exist.
#define O_TEXT    0x4000                          // (not supported) CR-LF in file becomes LF in memory.
#define O_BINARY  0x8000                          // (mandatory) Input and output is not translated.
#define O_RAW     O_BINARY

// error code (EOF)
#define FERROR -1                                 // values returned by file function if error

// seek modes
#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2

// ----------------------------------------------
// other structs

struct TDateTime
{
  u8 Hour;
  u8 Min;
  u8 Sec;
  u8 DayL;                                       // 1 if daylight savings time option set
  u16 Year;
  u8 Month;
  u8 Day;
};

// ----------------------------------------------
// va args
#define _INTSZ(n)   ( (sizeof(n) + sizeof(s32) - 1) & ~(sizeof(s32) - 1) )

typedef char *va_list;
#define va_start(ap,v)  ( ap = (va_list)&v + _INTSZ(v) )
#define va_arg(ap,t)    ( *(t *)((ap += _INTSZ(t)) - _INTSZ(t)) )
#define va_end(ap)      ( ap = (va_list)0 )

// ----------------------------------------------
// kernel

// event flags returned by WaitForEvent
#define EVT_SHUTDOWN 1                           // shutdown event
#define EVT_CLOSEAPP 2                           // close application event
#define EVT_FRAMEBUF 4                           // frame buffers have changed (can be size of bpp)
#define EVT_LOSTFOCUS 8                          // system have lost the focus (special for emulator into a window)

// base list of functions provided by kernel for user applications.
// for other functions, use GetProcAddress
struct TKernelLib
{
  // time functions
  struct
  {
    u32 TSC_One_ms;                              // TSC increment for 1ms
    u64 (*ReadTsc)(void);                        // TSC: a timer at cpu freq
    u32 (*GetTickCount)(void);                   // return ms tick counter
    void (*Sleep)(u32 ms);                       // wait in power saving mode (accuracy is about 1ms, time max limited to 5s to avoid blocking system in case of value error)
    bool (*getRTCdatetime)(struct TDateTime *DateTime);  // return time and date from RTC
  } Time;

  // memory
  struct
  {
    void *(*Malloc)(s32 ByteSize);
    void *(*Realloc)(void *Ptr, s32 ByteSize);
    void (*Free)(void *Ptr);
    void (*FreeAll)(void);                       // free everything allocated
  } Mem;

  // display
  struct
  {
    void (*PutChar)(s32 c);
    s32 (*Print)(const char *str);
    s32 (*Printf)(const char *fmt, ...);
    s32 (*vPrintf)(const char *fmt, va_list ap);
    void (*ClearScreen)(void);

    // frame buffer modes
    void *(*GetFrameBuffer)(s32 *x, s32 *y, s32 *Bpp, s32 *FBReserved, void **pVRam);  // return FB screen ptr and informations. pVRam is null under windows
    void (*BlitAll)(void);
    void (*BlitRect)(s32 x, s32 y, s32 dx, s32 dy);
    void (*BlitColor)(s32 x, s32 y, s32 dx, s32 dy, s32 Color);
  } Disp;

  // keyboard
  struct
  {
    bool (*KeyStates)(enum eKeyState KeyId);
    s32 (*KbdHit)(void);
    s32 (*GetKey)(void);

	  // define user call backs
	  void (*SetKeyDownCB)(void (*KeyDownCB)(u8 KeyCode));
	  void (*SetKeyUpCB)(void (*KeyUpCB)(u8 KeyCode));
  } Kbd;

  // mouse
  struct
  {
    bool (*Show)(bool Show);
    bool (*SetCursor)(enum ePosMouseCursor CursorShape);
    bool (*SetSpeed)(s32 Speed);
    s32 (*GetState)(s32 *x, s32 *y, s32 *w, s32 *butt);
    void (*RestoreEvent)(void);
  } Mouse;

  // files
  struct
  {
    // --------------------------------
    // errors for files operations
    s32 *ErrNo;                                      // ptr on error code
    const char *(*StrError)(s32 ErrCode);            // convert error code to string message

    // --------------------------------
    // file operation using file descriptorn. all return -1 if error
    s32 (*Open)(const char *fname, s32 mode, ...);   // open a file, return -1 if error, else file id (can be 0)
    s32 (*Close)(s32 fd);                            // close a file, retun -1 if error 0 is success
    s32 (*Lseek)(s32 fd, u32 ofs, s32 mode);         // seek to file position, return -1 if error, else seek position
    s32 (*Read)(s32 fd, void *buff, u32 sz);         // read in file, return -1 if error, else read size
    s32 (*Write)(s32 fd, const void *dat, u32 sz);   // write to file, return -1 if error, else write size
    s32 (*Tell)(s32 fd);                             // get file pointer position, return -1 if error, else position
    s32 (*Filelength)(s32 fd);                       // get file length, return -1 if error, else length
    s32 (*Fileno)(hFILE *f);                         // return file id, -1 if not found
    hFILE *(*FdOpen)(s32 fd, const char *Mode);      // open file using its descriptor

    // --------------------------------
    // file operation using file handle
    hFILE *(*Fopen)(const char *fname, const char *mode); // return NULL if fail
    s32 (*Fclose)(hFILE *f);                         // close a file, retun -1 if error 0 is success
    s32 (*Fseek)(hFILE *f, u32 ofs, s32 mode);       // seek to file position, return -1 if error, else seek position
    s32 (*Ftell)(hFILE *f);                          // return file position, return -1 if error
    s32 (*Filesize)(hFILE *f);                       // return file size, return -1 if error
    u32 (*Fread)(void *buff, u32 sz, hFILE *f);      // read in file, return read size, set ErrNo if error
    u32 (*Fwrite)(const void *buff, u32 sz, hFILE *f); // write to file, return written size, set ErrNo if error

    // --------------------------------
    // other file operations
    bool (*Fdel)(const char *Name);              // delete file, or empty directory
    bool (*Fmkdir)(const char *Name);            // create directory
    bool (*Frename)(const char *OldName, const char *NewName); // rename file / dir
    bool (*Fgetfree)(const char *VolName, u64 *TotalBytes, u64 *FreeBytes); // get volume free/used size
    void (*Fcloseall)(void);                     // close all opened files

    hFFIND *(*FFindFirst)(const char *Path, struct TFileInfo *FileInfo);
    bool (*FFindNext)(hFFIND *ff, struct TFileInfo *FileInfo);
    bool (*FFindClose)(hFFIND *ff);
    
    char *(*GetAbsPath)(const char *inPath, char *outPath, s32 outSize);  // return absolute path from relative path
    const char *(*GetExePath)(void);             // return path of currently executed program (without name)
    char *(*GetCwd)(char *buff, s32 sz);         // get current directory string, return buff if success, else NULL
  } Files;

  // miscelaneous
  struct
  {
    // errors handling
    void (*Exit)(s32 ExitCode);                  // exit program and pass exit code

    // jumps
    s32 (*SetJmp)(void *JmpBuf);                 // set jump position, return 0
    void (*LongJmp)(void *JmpBuf, s32 Code);     // jump to saved position, return code

    // wait
    u32 (*WaitForEvent)(void);                   // enter power saving until hardware event occur (from timer, keyboard, mouse..)

    // string functions
    s32 (*sPrintf)(char *buf, const char *fmt, ...);
    s32 (*snPrintf)(char *buf, s32 bufsz, const char *fmt, ...);
    s32 (*vsPrintf)(char *buf, const char *fmt, va_list ap);
    s32 (*vsnPrintf)(char *buf, s32 bufsz, const char *fmt, va_list ap);

    // extensions, return proc address
    void *(*GetProcAddress)(const char *ProcName);
  } Misc;
};

extern struct TKernelLib Kernel;

#ifdef  __cplusplus
}
#endif
