// this code contain functions used to compile tcc.

#include "dtype.h"
#include "kerlib.h"

#ifdef _INC_STDIO
#error ERROR: stdio.h cannot be used in POS application.
#endif

#define EPERM           1
#define ENOENT          2
#define ESRCH           3
#define EINTR           4
#define EIO             5
#define ENXIO           6
#define E2BIG           7
#define ENOEXEC         8
#define EBADF           9
#define ECHILD          10
#define EAGAIN          11
#define ENOMEM          12
#define EACCES          13
#define EFAULT          14
#define EBUSY           16
#define EEXIST          17
#define EXDEV           18
#define ENODEV          19
#define ENOTDIR         20
#define EISDIR          21
#define EINVAL          22
#define ENFILE          23
#define EMFILE          24
#define ENOTTY          25
#define EFBIG           27
#define ENOSPC          28
#define ESPIPE          29
#define EROFS           30
#define EMLINK          31
#define EPIPE           32
#define EDOM            33
#define ERANGE          34
#define EDEADLK         36
#define ENAMETOOLONG    38
#define ENOLCK          39
#define ENOSYS          40
#define ENOTEMPTY       41
#define EILSEQ          42

#define _JBLEN 16               // note: set enough for ms compiler if launched on emulator
#define _JBTYPE int
typedef _JBTYPE jmp_buf[_JBLEN];

#define FILE hFILE
typedef unsigned int size_t;
#ifndef NULL
#define NULL (void *)0
#endif

extern FILE *stdin;
extern FILE *stdout;
extern FILE *stderr;

#define _fileno fileno
#define _snprintf snprintf
#define _vsnprintf vsnprintf

#define strcasecmp stricmp
#define strncasecmp strnicmp

#if 1
  #define errno err_no
  extern int err_no;
#else
  #define errno (*Kernel.Files.ErrNo)
#endif

// ----------------------------------------------
// kernel specific functions

// direct mapped
#define exit       Kernel.Misc.Exit
#define putchar    Kernel.Disp.PutChar
#define printf     Kernel.Disp.Printf
#define vprintf    Kernel.Disp.vPrintf
#define sprintf    Kernel.Misc.sPrintf
#define snprintf   Kernel.Misc.snPrintf
#define vsprintf   Kernel.Misc.vsPrintf
#define vsnprintf  Kernel.Misc.vsnPrintf
#define malloc     Kernel.Mem.Malloc
#define realloc    Kernel.Mem.Realloc
#define free       Kernel.Mem.Free
#define setjmp     Kernel.Misc.SetJmp
#define longjmp    Kernel.Misc.LongJmp
#define getcwd     Kernel.Files.GetCwd

// call to sub function
#define calloc     _Calloc
#define open       _Open
#define close      _Close
#define lseek      _Lseek
#define read       _Read
#define write      _Write
#define tell       _Tell
#define filelength _Filelength
#define fileno     _Fileno
#define dup        _Dup
#define fdopen     _Fdopen
#define fopen      _Fopen
#define fclose     _Fclose
#define fseek      _Fseek
#define ftell      _Ftell
#define filesize   _Filesize
#define fread      _Fread
#define fwrite     _Fwrite
#define fprintf    _Fprintf
#define getc       _Getc
#define fgetc      _Getc
#define putc       _Putc
#define fputc      _Putc
#define fputs      _Fputs
#define fgets      _Fgets
#define strerror   _Strerror

void  *_Calloc(size_t n, size_t sz);

s32    _Open(const char *fname, s32 mode, ...);
s32    _Close(s32 fd);
long   _Lseek(s32 fd, long ofs, s32 mode);
s32    _Read(s32 fd, void *buff, u32 sz);
s32    _Write(s32 fd, const void *dat, u32 sz);
long   _Tell(s32 fd);
long   _Filelength(s32 fd);
s32    _Fileno(FILE *f);
s32    _Dup(s32 fid);
FILE * _Fdopen(s32 fd, const char *mode);
FILE * _Fopen(const char *fname, const char *mode);
s32    _Fclose(FILE *f);
s32    _Fseek(FILE *f, long ofs, s32 mode);
long   _Ftell(FILE *f);
long   _Filesize(FILE *f);
size_t _Fread(void *buff, size_t sz, size_t cnt, FILE *f);
size_t _Fwrite(const void *buff, size_t sz, size_t cnt, FILE *f);
s32    _Fprintf(FILE *f, const char *fmt, ...);
s32    _Getc(FILE *f);
s32    _Putc(s32 c, FILE *f);
s32    _Fputs(const char *s, FILE *f);
char  *_Fgets(char *s, s32 n, FILE *f);
char  *_Strerror(s32 err);
s32	   _Setjmp(jmp_buf env);
void   _Longjmp(jmp_buf env, s32 val);

// ----------------------------------------------
// numeric string conversion

#ifdef _ADDSTO_
#define strtol   _Strtol
#define strtoul  _Strtoul
#define strtoll  _Strtoll
#define strtoull _Strtoull
#define strtod   _Strtod
#define strtold  _Strtold
#define strtof   _Strtof
#define ldexp    _Ldexp

s32    _Strtol(const char *nptr, char **endptr, int base);
u32    _Strtoul(const char *nptr, char **endptr, int base);
s64    _Strtoll(const char *nptr, char **endptr, int base);
u64    _Strtoull(const char *nptr, char **endptr, int base);
double _Strtod(const char *nptr, char **endptr);
long double _Strtold(const char *nptr, char **endptr);
float  _Strtof(const char *nptr, char **endptr);
double _Ldexp(double x, int exp);
#endif

// ----------------------------------------------
// standard functions
#ifdef _ADDSTD_
#define strcmp   _Strcmp
#define stricmp  _Stricmp
#define strnicmp _Strnicmp
#define strchr   _Strchr
#define strrchr  _Strrchr
#define strncmp  _Strncmp
#define strlen   _Strlen
#define strcpy   _Strcpy
#define memset   _Memset
#define memcpy   _Memcpy
#define memcmp   _Memcmp
#define memmove  _Memmove
#define qsort    _Qsort

int    _Strncmp(const char *str0, const char *str1, size_t sz);
s32    _Stricmp(const char *str0, const char *str1);
s32    _Strnicmp(const char *str0, const char *str1, size_t sz);
char  *_Strchr(const char *str, int c);
char  *_Strrchr(const char *str, int c);
s32    _Strcmp(const char *str0, const char *str1);
unsigned long _Strlen(const char *str);
char  *_Strcpy(char *dest, const char *src);
void  *_Memset(void *dest, int v, size_t sz);
void  *_Memcpy(void *to, const void *from, size_t n);
s32    _Memcmp(const void *buff0, const void *buff1, size_t size);
void  *_Memmove(void *dest, const void *src, size_t size);
void   _Qsort(void *ObjList, size_t ObjCount, size_t ObjSizeof, s32 (*CB_Sort)(const void *, const void *));
#endif // _ADDSTD_