// some simple drawing function depending of colour depth
#include "pos.h"
#include "grproc.h"

struct FbInf Fb;
void (*DrawPixel)(int x, int y, int Color);
u32 (*GetColor)(u8 R, u8 G, u8 B);

// test if pixel is into screen
#define InScreen(x, y) (x >= 0) && (x < Fb.x) && (y >= 0) && (y < Fb.y)

// RGB 233
static u32 GetColor8(u8 R, u8 G, u8 B)
{
  return (B >> 5) | ((G & 0xe0) >> 2) | (R & 0xc0);
}

// RGB 565
static u32 GetColor16(u8 R, u8 G, u8 B)
{
  return ((B & 0xf8) >> 3) | ((G & 0xfc) << 3) | ((R & 0xf8) << 8);
}

// RGB 888
static u32 GetColor32(u8 R, u8 G, u8 B)
{
  return B | (G << 8) | (R << 16);
}

// color in RGB 233
static void DrawPixel8(int x, int y, int Color)
{
  if (InScreen(x, y))
  {
    u8 *p = (u8 *)Fb.pScreen;
    p[y*Fb.x + x] = (u8)Color;
  }
}

// color in RGB 565
static void DrawPixel16(int x, int y, int Color)
{
  if (InScreen(x, y))
  {
    u16 *p = (u16 *)Fb.pScreen;
    p[y*Fb.x + x] = (u16)Color;
  }
}

// color in RGB 888
static void DrawPixel32(int x, int y, int Color)
{
  if (InScreen(x, y))
  {
    u32 *p = (u32 *)Fb.pScreen;
    p[y*Fb.x + x] = (u32)Color;
  }
}

static u8 ClipXY(s32 *x0, s32 *y0, s32 *x1, s32 *y1)
{
  if (*x0 < 0)
    *x0 = 0;
  if (*y0 < 0)
    *y0 = 0;
  if (*x1 > Fb.x)
    *x1 = Fb.x;
  if (*y1 > Fb.y)
    *y1 = Fb.y;
  return (*x1 > *x0) && (*y1 > *y0);
}

void ClearFb(vec2i *Pos, vec2i *Size, int Color)
{
  s32 x0, y0, x1, y1;
  x0 = Pos->x;
  y0 = Pos->y;
  x1 = Pos->x + Size->x;
  y1 = Pos->y + Size->y;

  if (ClipXY(&x0, &y0, &x1, &y1))
  {
    int wx = x1 - x0;

    if (Fb.Bpp == 8)
    {
      u8 Color8 = (u8)Color;
      u8 *p = (u8 *)Fb.pScreen;
      u8 *D = &p[x0 + y0*Fb.x];
      while (y0 < y1)
      {
        const u8 *DEnd = &D[wx];
        u8 *D8 = D;

        while (D8 < DEnd)
          *(D8++) = Color8;

        D += Fb.x;
        y0++;
      }
    }
    else 
    if (Fb.Bpp == 16)
    {
      u16 Color16 = (u16)Color;
      u16 *p = (u16 *)Fb.pScreen;
      u16 *D = &p[x0 + y0*Fb.x];
      while (y0 < y1)
      {
        const u16 *DEnd = &D[wx];
        u16 *D16 = D;

        while (D16 < DEnd)
          *(D16++) = Color16;

        D += Fb.x;
        y0++;
      }
    }
    else
    if (Fb.Bpp == 32)
    {
      u32 *p = (u32 *)Fb.pScreen;
      u32 *D = &p[x0 + y0*Fb.x];
      while (y0 < y1)
      {
        const u32 *DEnd = &D[wx];
        u32 *D32 = D;

        while (D32 < DEnd)
          *(D32++) = Color;

        D += Fb.x;
        y0++;
      }
    }
  }
}

u8 InitGrProc(void)
{
  DrawPixel = NULL;

  // get frame buffer infos
  Fb.pScreen = Kernel.Disp.GetFrameBuffer(&Fb.x, &Fb.y, &Fb.Bpp, &Fb.FBReserved, &Fb.pVRam);

  if (Fb.pScreen)
  {
    if (Fb.Bpp == 8)
    {
      DrawPixel = DrawPixel8;
      GetColor = GetColor8;
    }
    else
    if (Fb.Bpp == 16)
    {
      DrawPixel = DrawPixel16;
      GetColor = GetColor16;
    }
    else
    if (Fb.Bpp == 32)
    {
      DrawPixel = DrawPixel32;
      GetColor = GetColor32;
    }
  }

  if (!DrawPixel)
  {
    printf("ERROR: no suitable graphic mode found.\n");  // we are in text mode ?
    return 0;
  }
  return 1;
}
